##################################################################
proc d2n {input} {

	global objname
	global objdot

	regsub -all {\.} $input " " idlist
	set numofids [llength $idlist]
	set index -1
	for { set i $numofids } { $i > 0 } { incr i -1} {
		set temptarget ""
		for { set j 0 } { $j < $i } { incr j } {
			if {$j!=0} { append temptarget "." }
			append temptarget [lindex $idlist $j]
		}
		set index [lsearch -exact $objdot $temptarget]
		if { $index > 0 } break 
	}
	if { $index > 0 } {
		set result [lindex $objname $index]
		for { set j $i } { $j < $numofids } { incr j } {
			append result "."
			append result [lindex $idlist $j]
		}
		return $result 
	} else {
		return "not- match" 
	}
}


##################################################################
proc n2d {input} {

	global objname
	global objdot

	set index [ string first "." $input ]
	if { $index < 0 } {
		set tindex [lsearch $objname $input]
		if { $tindex > 0 } {
			set result [lindex $objdot $tindex]
			return $result
		} else {
			return "not-match" 
		}
	} else {
		set namepart [string range $input 0 [expr $index-1 ] ]
		set dotpart  [string range $input $index end]
		if { [regexp {^[0-9\.]*$} $dotpart ] == 0 } {
			return "bad-format"
		} else {
			set tindex [lsearch $objname $namepart]
			if { $tindex > 0 } {
				set result [lindex $objdot $tindex]
				set alldot $result
				append alldot $dotpart				
				return $alldot
			} else {
				return "not-match" 
			}
		}
	}
}


##################################################################
proc putoutdsc { prefix1 prefix2 description length } {
	
	set sindex 0
	set eindex 0
	set first "yes"
	
	while { $sindex < [string length $description] } {
		set eindex [string wordstart $description [expr $sindex+$length] ]
		if { [string index $description $eindex] == "." || [string index $description $eindex] == "," } {
			incr eindex
		}
		if { $sindex == $eindex } break
		set rangetext [string range $description $sindex [expr $eindex-1] ]
		if { $first == "yes" } {
			set output $prefix1
			set first "no"
		} else {
			set output $prefix2
		}
		append output $rangetext
		puts stdout $output
		set sindex [expr $eindex]
	}

	return
}

# mib file to cnf source convert tool
# 
#   input parm1(mib\[Xt@C)  parm2(SystemwalkerCXg[fBNg)
# 
#   output Wo (cnf\[X)
#   
#      ϐ  allsource  c  Smib\[X̓ǂݍ
#                objname    c  Sobj-namelist
#                objdot     c  Sobjidlist
#                objtype    c  Sobǰ^list
#                
#                objdisc    c  objdescriptionlist. discꍇobj-name.
#                objvdsc    c  obj̒ldescription list. discꍇobj-name.
#                

	
	variable objname ""
	variable objdot ""

	# Operands check
	if { $argc != 2 } {
		puts stderr "ERROR Invalid parameter -- xxxxx mib-source(*.my) mib-file(*.mib)"
		exit
	}
	set SourceFileName [lindex $argv 0]
	set MibDirName [lindex $argv 1]\\mpwalker.dm\\mib
	set FileTale [ file tail $SourceFileName ]
	regsub "\.my$" $FileTale "\.mib" FileTale
	set MibFileName ${MibDirName}\\$FileTale

	if { [ file readable $SourceFileName ] == 0 } {
		puts stderr "ERROR $SourceFileName is not readable"
		exit
	}
	if { [ file exists $MibDirName ] == 0 } {
		puts stderr "ERROR $MibDirName does not exist"
		exit
	}
	if { [ file readable $MibFileName ] == 0 } {
		puts stderr "ERROR $SourceFileName is not compiled"
		exit
	}

	puts stderr "Initialize complete."

	# Read all source text except comment, and reformat test (remove nl,cr,tab,etc).
	puts -nonewline stderr "Mib Source Text reading."
	set fileid [ open $SourceFileName "r" ]
	set allsource [ read -nonewline $fileid ]
	close $fileid
	puts -nonewline stderr "."

	regsub -all -- {--[^\n]*\n} $allsource " " allsource
	puts -nonewline stderr "."
	regsub -all {[\r\n\t]} $allsource " " allsource
#	regsub -all "\r" $allsource " " allsource
#	puts -nonewline stderr "."
#	regsub -all "\n" $allsource " " allsource
#	puts -nonewline stderr "."
#	regsub -all "\t" $allsource " " allsource
	puts -nonewline stderr "."
	regsub -all "::=" $allsource " ::= " allsource
	puts -nonewline stderr "."
#	regsub -all " +" $allsource " " allsource
#	puts -nonewline stderr "."
	regsub -all "OCTET STRING" $allsource "OCTET-STRING" allsource
	puts -nonewline stderr "."
	regsub -all "OBJECT IDENTIFIER" $allsource "OBJECT-IDENTIFIER" allsource
	puts -nonewline stderr "."
	regsub -all "SEQUENCE OF" $allsource "SEQUENCE-OF" allsource
	puts -nonewline stderr "."
	puts stderr ".complete."

	# Read xxx.MIB file, and initialize obj name list, obj dot-format-id list, and obj type list.
	puts -nonewline stderr "Mib file reading."
	# path separators are converted platform separator to tcl separator
	set MibDirName [file join ${MibDirName}]

	# getting mib file list
	set mibfiles [glob -path ${MibDirName}/ {*.[mM][iI][bB]}]

	foreach file ${mibfiles} {
		# Read xxx.MIB file,
		# and initialize obj name list and obj dot-format-id list.
		set mfileid [open "${file}" r]
		while { [eof ${mfileid}] == 0 } {
			gets ${mfileid} record
			regsub -all "," ${record} " " record
			if { [lsearch -exact $objname [lindex ${record} 0] ] } {
				lappend objname [lindex $record 0]
				lappend objdot  [lindex $record 1]
				lappend objtype [lindex $record 2]
			}
		}
		close ${mfileid}
		puts stdout " file : ${file} processed."
	}
	puts stderr "..complete."

	# Initialize obj description list.
	set objdisc $objname
	set objvdsc $objname

	# First pass. Search all source and set obj description text to objdisc list.
	for { set i 0 } { $i < [llength $allsource] } { incr i } {
		set Keyword [lindex $allsource $i]
		if { $Keyword == "IMPORTS" || $Keyword == "EXPORTS" } then {
			# IMPORTS macro or EXPORTS macro. Skip to ";".
			while { [regexp {;$} [lindex $allsource $i] ] == 0 } { incr i}
			continue
		} elseif { $Keyword == "OBJECT-TYPE" } then {
			# Object is found.
			set objectname [lindex $allsource  [expr $i-1] ]
			while { [lindex $allsource $i] != "SYNTAX" } { incr i}
			set objectvdsc [lindex $allsource  [expr $i+2] ]
			while { [lindex $allsource $i] != "DESCRIPTION" } { incr i}
			set objectdisc [lindex $allsource  [expr $i+1] ]
			set objindex [lsearch $objname $objectname]
			set objvdsc [lreplace $objvdsc $objindex $objindex $objectvdsc ]
			set objdisc [lreplace $objdisc $objindex $objindex $objectdisc ]
		}
	}

	# Second pass. Search all source and find TRAP-TYPE definition. And output text.
	for { set i 0 } { $i < [llength $allsource] } { incr i } {
		set Keyword [lindex $allsource $i]
		if { $Keyword == "IMPORTS" || $Keyword == "EXPORTS" } then {
			# IMPORTS macro or EXPORTS macro. Skip to ";".
			while { [regexp {;$} [lindex $allsource $i] ] == 0 } { incr i}
			continue
		} elseif { $Keyword == "TRAP-TYPE" } then {
			# TRAP definition is found.
			set trapname [lindex $allsource  [expr $i-1] ]
			while { [lindex $allsource $i] != "ENTERPRISE" } { incr i}
			set enterprise [lindex $allsource  [expr $i+1] ]
			set variables ""
			set description ""
			while { [lindex $allsource $i] != "VARIABLES" \
			     && [lindex $allsource $i] != "DESCRIPTION" \
			     && [lindex $allsource $i] != "::="  } { incr i}
			if { [lindex $allsource $i] == "VARIABLES" } then {
				set variables [lindex $allsource  [expr $i+1] ]
				while { [lindex $allsource $i] != "DESCRIPTION" \
				     && [lindex $allsource $i] != "::="  } { incr i}
				if  { [lindex $allsource $i] == "DESCRIPTION" } then {
					set description [lindex $allsource  [expr $i+1] ]
					while { [lindex $allsource $i] != "::="  } { incr i}
					set specificcode [lindex $allsource  [expr $i+1] ]
				} else {
					set specificcode [lindex $allsource  [expr $i+1] ]
				}
			} elseif { [lindex $allsource $i] == "DESCRIPTION" } then {
				set description [lindex $allsource  [expr $i+1] ]
				while { [lindex $allsource $i] != "VARIABLES" \
				     && [lindex $allsource $i] != "::="  } { incr i}
				if  { [lindex $allsource $i] == "VARIABLES" } then {
					set variables [lindex $allsource  [expr $i+1] ]
					while { [lindex $allsource $i] != "::="  } { incr i}
					set specificcode [lindex $allsource  [expr $i+1] ]
				} else {
					set specificcode [lindex $allsource  [expr $i+1] ]
				}
			} else {
				set specificcode [lindex $allsource  [expr $i+1] ]
			}
			# Information collection complete.
			regsub -all " +" $description " " description

			puts stdout "## $trapname ( Enterprise=$enterprise [n2d $enterprise] ) definition begin"
			puts stdout "# TRAP-NAME     : $trapname"
			puts stdout "#  Enterprises  : $enterprise ([n2d $enterprise])"
			puts stdout "#  Specific     : $specificcode"
			putoutdsc   "#  description  : " "#                 " $description 55
			
			if { $variables != "" } {
				regsub -all "," $variables " " variables
				regsub -all " +" $variables " " variables
				putoutdsc   "#  Variables    : " "#                 " $variables  55
			}
			puts stdout "# "
			for { set j 0 } { $j < [llength $variables] } { incr j } {
				set  varname [lindex $variables $j]
				regsub  ",$" $varname "" varname
				puts stdout "#        | $varname information"
				set objindex [lsearch $objname $varname]
				set vartype  [lindex  $objtype $objindex]
				if { $vartype == "i" } then {
					set vartype "Integer" 
				} elseif { $vartype == "i" } then { 
					set vartype "Integer" 
				} elseif { $vartype == "d" } then { 
					set vartype "Object ID" 
				} elseif { $vartype == "o" } then { 
					set vartype "Octet String" 
				} elseif { $vartype == "a" } then { 
					set vartype "IP Address" 
				} elseif { $vartype == "c" } then { 
					set vartype "Counter" 
				} elseif { $vartype == "g" } then { 
					set vartype "Guage" 
				} elseif { $vartype == "t" } then { 
					set vartype "Timeticks" 
				} else   {
					set vartype "Aggregate" 
				}
				set vardot   [lindex  $objdot  $objindex]
				set vartypesuff  [lindex  $objvdsc $objindex]
				set vardisc  [lindex  $objdisc $objindex] 
				puts stdout "#              id          : $vardot"
				puts stdout "#              type        : $vartype"
				if { $vartypesuff != "ACCESS" } {
					regsub -all " +" $vartypesuff " " vartypesuff
#					puts stdout "#                            $vartypesuff"
					putoutdsc "#                              " "#                              " $vartypesuff 38
				}
				regsub -all " +" $vardisc " " vardisc
				putoutdsc "#              description : " "#                            " $vardisc 40
				puts stdout "#"
			}
		puts stdout "#"
		puts stdout "#"
		puts stdout "##!! Warning !! You should add Japanese and English message texts."
		puts stdout "TRAP:"
		puts stdout " - 4 - - [n2d $enterprise] - 6 $specificcode - "
		puts stdout "VARBIND:"
		puts stdout " - - - -"
		puts stdout "MESSAGE:"
		puts stdout "SmallJ:"
		puts stdout " \"\[%%hostx:%AGENT_ADDR%\]$trapname gbvM܂. \" " 
		puts stdout "SmallE:"
		puts stdout " \"\[%%hostx:%AGENT_ADDR%\]Trap ($trapname) receieved. \" " 
		puts stdout "##!! Warning !! You should check EventType parameter."
		puts stdout "EVENTTYPE:"
		puts stdout " ERROR"
#		puts stdout "Source:"
#		puts stdout " MpCNappl"
		puts stdout "##!! Warning !! You should check Treshold parameter."
		puts stdout "Threshold:"
		puts stdout " 1"
		puts stdout "## $trapname ( Enterprise=$enterprise ) definition end "
		puts stdout ""
		puts stdout ""
		
		continue
		}
	}

