#
# cnf file syntax checker
# 
#   input : parm1(cnf file name)
#          [parm2( C/DC/DCT C:only checking default DC:detail check  DCT:check and outputtestdata )]
#          [parm3(Systemwalker install directory)]
# 
#          If parm2 is "DC/DCT", parm3 must be specified
#
#   output: STDERR ( ERROR/WARNING messages )
#           STDOUT (test data)
#   
#   code  : 0 : no error 
#           4 : Warning message exist
#           8 : Error
#          16 : Error, unable to continue
#   
#   Internal Spec
#
#    status -- WFIRST
#           -- WCMNT
#           -- WTRAP
#           -- WTRAPDATA
#           -- WVARBIND
#           -- WVARBINDDATA
#           -- WVARBINDDATAORMESSAGE
#           -- WEJMSG
#           -- WEMSGDATAF
#           -- WJMSGDATAF
#           -- WEMSGL
#           -- WJMSGL
#           -- WEMSGDATAL
#           -- WJMSGDATAL
#           -- WEVENTTYPE
#           -- WEVENTTYPEDATA
#           -- WSOURCE
#           -- WSOURCEDATA
#           -- WTHRESHOLD
#           -- WTHRESHOLDDATA
#           -- ERRSTAT
#
#    procSW -- C   : Check
#           -- DC  : Dtail Check
#           -- DCT : Dtail Check with Test Code 
#
#
#
#

# 
# putoutdsc
# 
proc putoutdsc { prefix1 prefix2 description length } {
	
	set sindex 0
	set eindex 0
	set first "yes"
	
	while { $sindex < [string length $description] } {
		set eindex [string wordstart $description [expr $sindex+$length] ]
		if { [string index $description $eindex] == "." ||
		     [string index $description $eindex] == "," ||
		     [string index $description $eindex] == ")" } {
			incr eindex
		}
		if { $sindex == $eindex } break
		set rangetext [string range $description $sindex [expr $eindex-1] ]
		if { $first == "yes" } {
			set output $prefix1
			set first "no"
		} else {
			set output $prefix2
		}
		append output $rangetext
		puts stderr $output
		set sindex [expr $eindex]
	}

	return
}

# 
# procERROR
# 
proc procERROR { text } {
	global status

	putoutdsc "Error   (Source: [Rn]) F " "                          " $text 50
	
	set status "ERRSTAT"
	return 8
}



# 
# procSERROR
# 
proc procSERROR { text } {
	global status

	putoutdsc "S-Error (Source: [Rn]) F " "                          " $text 50
	
	set status "ERRSTAT"
	return 16
}


# 
# procWARN
# 
proc procWARN { text } {
	global status

	putoutdsc "Warning (Source: [Rn]) F " "                          " $text 50

	return 4
}


# 
# procWFIRST
# 
proc procWFIRST { record } {
	global Enterprise procSW status topOID topOIDdot

	if { [regexp {^# *enterprises *=} $record ] == 1} then {
		regsub -all "=" $record " " record
		if { [llength $record] < 3 } {
			return [procSERROR "No enterprise OID specified. First Record must be \"# enterprises=<enterprise OID>\" "]
		}
		set Enterprise [lindex $record 2]
	} else {
		return [procSERROR "No enterprise OID specified. First Record must be \"# enterprises=<enterprise OID>\" " ]
	}
	if { [ isExistMib $Enterprise ] == "INV" } then {
		return [procSERROR "Specified enterprise OID is invalid in format."]
	}
	if { $procSW == "C" } then {
		set status "WCMNT"
		return [procWARN "You should check the representation of Enterprise-OID in your target Systemwalker system." ]
	} else {
		if { [ isExistMib $Enterprise ] == "UKN" } then {
 			if { [ isAllDot $Enterprise ] == "DOT" } then {
				set topOID "Dot-Nexist"
				set status "WCMNT"
				return [procWARN "Specified dot-notation OID is not defined in Systemwalker." ]
			} else {
				set topOID "Ndot-Nexist"
				return [procERROR "Specified OID is not defined in Systemwalker." ]
			}
		}
		set topOID "Exist"
		set topOIDdot [n2d $Enterprise]
		set status "WCMNT"
	} 
	
	return 0
}

		
# 
# procWCMNT
# 
proc procWCMNT { record } {
	global status

	set status "WTRAP"
	return 0
}


# 
# procWTRAP
# 
proc procWTRAP { record } {
	global status Sourceline

	set Sourceline [Rn]
	if { [regexp "^TRAP:" $record ] != 1} {
		return [procERROR "Invalid keyword. Definition must start with \"TRAP:\"."]
	}
 	if { [llength $record] > 1 } {
		return [procERROR "Invalid parameter specified. Keyword's parameter must be in the following record." ]
	}

	set status "WTRAPDATA"
	return 0
}


# 
# procWTRAPDATA
# 
proc procWTRAPDATA { record } {
	global Enterprise status Community procSW Oid topOID topOIDdot Agent Generic Specific

	set rcode 0
 	if { [llength $record] != 9 } {
		return [procERROR "Invalid parameters. 9 parameters required. " ]
	}
 	if { [lindex $record 0] != "-" } then {
		set rcode [Max $rcode [procWARN "First parameter (community) specified. It's better to specify \"-\" to match the all traps."] ]
		set Community [lindex $record 0]
	} else { 
		set Community "public"
	}
	if { [lindex $record 1] != "-" && [lindex $record 1] != 4 } then {
		return [procERROR "2nd parameter must be \"-\" or \"4\"."]
 	}
	if { [lindex $record 2] != "-" && [lindex $record 2] != 0 } then {
		return [procERROR "3rd parameter must be \"-\" or \"0\"."]
	}
	if { [lindex $record 3] != "-" && [lindex $record 3] != 0 } then {
		return [procERROR "4th parameter must be \"-\" or \"0\"."]
	}
	if { [ isExistMib [lindex $record 4] ] == "INV" } then {
		return [procERROR "5th parameter (OID) is invalid in format."]
	}
	if { $procSW == "C" } then {
 		if { [lindex $record 4] != $Enterprise } then {
			if  { [isAllDot [lindex $record 4]] == "DOT" && [isAllDot $Enterprise]=="DOT" } then {
 				return [procERROR "5th parameter doesn't match the first record's one."]
			} else {
			 	set rcode [Max $rcode [procWARN "5th parameter (OID) doesn't match the first record's one."]]
			}
		}
		set Oid [lindex $record 4]
	} else {
		if { [ isExistMib [lindex $record 4] ] == "UKN" } then {
 			if { [ isAllDot [lindex $record 4] ] == "DOT" } then {
				if { $topOID == "Dot-Nexist" } then {
			 		if { [lindex $record 4] != $Enterprise } then {
		 				return [procERROR " 5th parameter (dot-notation OID) doesn't match the first record's one."]
 					}
				} elseif { $topOID == "Ndot-Nexist" } then {
					set rcode [Max $rcode [procWARN "Cannot check 5th parameter (dot-notation OID). You should check this parameter and the first record's one." ]]
				} else {
	 				return [procERROR "5th parameter (dot-notation OID) doesn't match the first record's one."]
				}
			} else {
				if { $topOID == "Dot-Nexist" } then {
					set rcode [Max $rcode [procWARN "Cannot check 5th parameter. You should check this parameter and the first record's one." ]]
				} elseif { $topOID == "Ndot-Nexist" } then {
			 		if { [lindex $record 4] != $Enterprise } then {
		 				return [procERROR "5th parameter doesn't match the first record's one."]
 					}
				} else {
	 				return [procERROR "5th parameter doesn't match the first record's one."]
				}
			}
			set Oid [lindex $record 4]
		} else {
			if { $topOIDdot != [n2d [lindex $record 4]] } then {
				return [procERROR "5th parameter doesn't match the first record's one." ]
			}
			set Oid [ n2d [lindex $record 4] ]
		}
	}
	if { [lindex $record 5] != "-" } then {
		if { [isIPA [lindex $record 5] ] == "" } then {
			return [procERROR "6th parameter (IP address) is invalid in format."]
		} 
		set rcode [Max $rcode [procWARN "6th parameter (IP address) specified. It's better to specify \"-\" to match the all traps." ]]
		set Agent [lindex $record 5]
	} else { 
		set Agent "%SOURCE%" 
	}
 	if { [ regexp {^[0-6\-]$} [lindex $record 6] ] !=1 } then {
		return [procERROR "7th parameter (Generic Trap code) must be 0,1,2,3,4,5,6, or -."]	
	} else { 
		set Generic [lindex $record 6]
	}
 	if { [ regexp {^-$} $Generic ] ==1 } then {
		set Generic 6
	}
# 	if { [ regexp {^[0-5]$} $Generic ] ==1 && [lindex $record 7] != "-" } then {
#		return [procERROR "8th parameter (Specific code) must be \"-\", if Generic code is 0,1,2,3,4,or 5."]
#	}
	if { [regexp {^[0-9][0-9]*$} [lindex $record 7]]!=1 && [lindex $record 7] != "-" } {
		return [procERROR "8th parameter (Specific code) must be numeric or \"-\"."]
 	} else { 
		set Specific [lindex $record 7] 
	}
	if { [lindex $record 8] != "-" } then {
		return [procERROR "9th parameter must be \"-\"."]
	}

	set status "WVARBIND"
	return $rcode
}
 	
# 
# procWVARBIND
# 
proc procWVARBIND { record } {
	global status VBINDEX

	if { [regexp "^VARBIND:" $record ] != 1} {
		return [procERROR "Invalid keyword. \"VARBIND:\" definition required."]
	}
 	if { [llength $record] > 1 } {
		return [procERROR "Invalid parameter specified. Keyword's parameter must be in the following record." ]
	}

 	set status "WVARBINDDATA"
 	set VBINDEX 0
 	return 0
}


# 
# procWVARBINDDATA
# 
proc procWVARBINDDATA { record } {
	global status Mib VBINDEX Mtype Maorb Mvalue procSW

	set rcode 0
	if { [regexp "^MESSAGE:" $record ] == 1} {
		if { $status == "WVARBINDDATA" } then {
			return [procERROR "There is no Varbind data. The data must be described after \"VARBIND:\" statement."]
		}
	 	if { [llength $record] > 1 } {
			return [procERROR " Invalid parameter specified. Keyword's parameter must be in the following record." ]
		}
		set status "WEJMSG"
 		return 0
	}

	if { [llength $record] < 4 } then {
		return [procERROR "Invalid parameters. 4 parameters required."]
	}
	set VB1 [lindex $record 0]  
	set VB2 [lindex $record 1]  
	set VB3 [lindex $record 2]  
	if { [llength $record] > 4 } then {
		# It May be Display string with Blank(s).
		if { [regexp {^\"} [lindex $record 3] ] !=1 ||  [regexp {.*\" *$} $record ] != 1 } then {
			return [procERROR "Invalid parameters. 4 parameters required. If 4th parameter is string, \" \" required."]
		}
		set VBV $record
		regsub $VB1  $VBV "" VBV
		regsub $VB2  $VBV "" VBV
		regsub $VB3  $VBV "" VBV
		regsub "^ *"  $VBV "" VBV
		regsub "^\""  $VBV "" VBV
		regsub "\"$"  $VBV "" VBV
	} else { 
		set VBV [lindex $record 3]
	}
	if { $VB1 != "-" && $VB2 != "-" && $VB3 != "-" && $VBV != "-" } then {
		if { [ isExistMib $VB1 ] == "INV" } then {
			return [procERROR "First parameter (MIB object) is invalid in format."]
		}
		if { $procSW == "C" } then {
			set rcode [Max $rcode [procWARN "First parameter (MIB object) specified. You should check MIB object name and instance."] ]
		} elseif { [ isExistMib $VB1 ] == "UKN" } then {
			set rcode [Max $rcode [procWARN "First parameter (MIB object) specified, but not defined in Systemwalker. You should check MIB object name and instance."] ]
		} elseif { [ isExistMib $VB1 ] == "UKNT" } then {
			# 2003/03/25 change logic by S.Nakagoshi
			#return [procERROR "First parameter is not MIB object. MIB object required."]
			set rcode [Max $rcode [procWARN "First parameter (MIB object) specified, but it's MIB table. You should check MIB object type."] ]
		} 
		set Mib($VBINDEX) [n2d $VB1]
		if { $procSW == "C" ||  [ isExistMib $VB1 ] == "UKN" } then {
	 		if { $VB2!="INT" && $VB2!="OCT" && $VB2!="OID" &&  $VB2!="IPA" && $VB2!="CNT" && $VB2!="GAG" && $VB2!="TIM" && $VB2!="OPA" } then {
				return [procERROR "2nd parameter invalid. Must be /INT/OCT/OID/IPA/CNT/GAG/TIM/OPA."]
			} else {
				set Mtype($VBINDEX) $VB2 
			}
		} else {
			if { $VB2 != [ isExistMib $VB1 ] } then {
				# 2003/02/27 change logic checkRecord by S.Nakagoshi
				procWARN "2nd parameter (MIB object type) does not match to first parameter."
			}
			set Mtype($VBINDEX) $VB2
		} 

		if { $VB3 != 0 && $VB3 != 1 } then {
			return [procERROR "3rd parameter must be 0 or 1."]
		} elseif { $VB3 == 1 && [expr { $VB2=="INT" || $VB2=="OID" || $VB2=="IPA" || $VB2=="CNT" || $VB2=="GAG" || $VB2=="TIM" }] } then {
			return [procERROR "3rd parameter must be 0 when 2nd parameter is INT/OID/IPA/CNT/GAG/TIM."]
		} else {
			set Maorb($VBINDEX) $VB3 
		}

 		if { [expr { $VB2=="INT" || $VB2=="CNT" || $VB2=="GAG" || $VB2=="TIM" } ] } then {
			# Integer/Counter/Gauge/Timeticks
 			if {  [regexp {^[0-9][0-9]*$} $VBV ]!=1 } then {
				return [procERROR "4th parameter must be numeric when 2nd parameter is INT/CNT/GAG/TIM. "]
 			}
			set Mvalue($VBINDEX) $VBV
 		} elseif { [expr { $VB2=="IPA" } ] } then {
			# IP Address
 			if {  [isIPA $VBV ] == "" } then {
				return [procERROR "4th parameter must be IP address (dot format) when 2nd parameter is IPA. "]
 			}
			set Mvalue($VBINDEX) $VBV
		}  elseif { [expr { $VB2=="OID"} ] } then {
			# OID
			if { [ isExistMib $VBV ] == "INV" } then {
				return [procERROR "4th parameter (object-id) is invalid in format."]
			}
			if { $procSW == "C" } then {
				# no check OID
				set rcode [Max $rcode [procWARN "4th parameter (object-id) specified. You should check object name."] ]
				set Mvalue($VBINDEX) $VBV
			} elseif { [ isExistMib $VB1 ] == "UKN" } then {
				# Unknown Mib object name
				set rcode [Max $rcode [procWARN "4th parameter (object-id) specified, but not defined in Systemwalker. You should check object name."] ]
				set Mvalue($VBINDEX) $VBV
			} else {
				# known Mib object name
				set Mvalue($VBINDEX) [n2d $VBV]
			} 
		} elseif { $VB3 == 1 } then {
			# binary flag
			foreach { hex } $VBV {
				if { [regexp -nocase {^[0-9A-F][0-9A-F]$} $hex ] != 1} then {
					return [procERROR "4th parameter is invalid in format. Must be a series of two digit hex-code separated by one blank."]
				}
			} 
			if { [regexp "  " $VBV ] == 1 || [regexp "^ " $VBV ] == 1 || [regexp " $" $VBV ] == 1 } then {
				return [procERROR "4th parameter is invalid in format. Must be a series of two digit hex-code separated by one blank."]
			}
			set Mvalue($VBINDEX) \"$VBV\"
		} else {
			#regsub -all {\\\{} $VBV \{ VBV
			#regsub -all {\\\}} $VBV \} VBV
			#regsub -all {\\"}  $VBV {""} VBV
			set Mvalue($VBINDEX) \"$VBV\"
		}
	} elseif { $VB1 != "-" || $VB2 != "-" || $VB3 != "-" || $VBV != "-"  } then {
		return [procERROR "These 4 parameters must be all \"-\" or all specified. "]
	} else {
 		set Mib($VBINDEX) "-"
 		set Mtype($VBINDEX) "-"
 		set Maorb($VBINDEX) "-"
 		set Mvalue($VBINDEX) "-"
 	}
	set status "WVARBINDDATAORMESSAGE"
	incr VBINDEX
	return $rcode
}

# 
# procWEJMSG
# 
proc procWEJMSG { record } {
	global status

	if { [regexp "^ *SmallJ:" $record ] == 1} then {
		set status "WJMSGDATAF"
	} elseif { [regexp "^ *SmallE:" $record ] == 1} then {
		set status "WEMSGDATAF"
	} else {
		return [procERROR "Invalid keyword. \"SmallJ\" or \"smallE\" definition required."]
	}
 	if { [llength $record] > 1 } {
		return [procERROR "Invalid parameter specified. Keyword's parameter must be in the following record." ]
	}

	return 0
}


# 
# procWEMSGDATAF
# 
proc procWEMSGDATAF { record } {
	global status EMsgText

	if { [regexp "^ *\{.*\} *$" $record] !=1 } then {
		return [procERROR "Invalid format. Message Text must be blank\"text\"."]
	} 
	set MsgText [lindex $record 0]

 	if { [string range $MsgText 0 21] != "\[%%hostx:%AGENT_ADDR%\]" } then {
		return [procERROR "Invalid message text. Message Text must be started with \"\[%%hostx:%AGENT_ADDR%\]\" ."]
	}
	if { [string is ascii $record ] == 0 } then {
		return [procERROR "Japanese character in English message text. "]
	}

	set status "WJMSGL"
	regsub -all {\\\{} $MsgText \{ MsgText
	regsub -all {\\\}} $MsgText \} MsgText
	regsub -all {\\"}  $MsgText {""} MsgText
	set EMsgText $MsgText
	return 0
}


# 
# procWJMSGDATAF
# 
proc procWJMSGDATAF { record } {
	global status JMsgText

	if { [regexp "^ *\{.*\} *$" $record] !=1 } then {
		return [procERROR "Invalid format. Message Text must be blank\"text\"."]
	} 
	set MsgText [lindex $record 0]

 	if { [string range $MsgText 0 21] != "\[%%hostx:%AGENT_ADDR%\]" } then {
		return [procERROR "Invalid message text. Message Text must be started with \"\[%%hostx:%AGENT_ADDR%\]\" ."]
	}

	set status "WEMSGL"
	regsub -all {\\\{} $MsgText \{ MsgText
	regsub -all {\\\}} $MsgText \} MsgText
	regsub -all {\\"}  $MsgText {""} MsgText
 	set JMsgText \"$MsgText\"
	return 0

}


# 
# procWEMSGL
# 
proc procWEMSGL { record } {
	global status

	if { [regexp "^ *SmallJ:" $record ] == 1} then {
		return [procERROR "Duplicate Japanese message."]
	} elseif { [regexp "^ *SmallE:" $record ] == 1} then {
		set status "WEMSGDATAL"
	} else {
		return [procERROR "Invalid keyword. \"SmallE\" definition required."]
	}

 	if { [llength $record] > 1 } {
		return [procERROR "Invalid parameter specified. Keyword's parameter must be in the following record." ]
	}

	return 0
}


# 
# procWJMSGL
# 
proc procWJMSGL { record } {
	global status

	if { [regexp "^ *SmallJ:" $record ] == 1} then {
		set status "WJMSGDATAL"
	} elseif { [regexp "^ *SmallE:" $record ] == 1} then {
		return [procERROR "Duplicate English message."]
	} else {
		return [procERROR "Invalid keyword. \"SmallJ\" definition required."]
	}
 	if { [llength $record] > 1 } {
		return [procERROR "Invalid parameter specified. Keyword's parameter must be in the following record." ]
	}

	return 0
}


# 
# procWEMSGDATAL
# 
proc procWEMSGDATAL { record } {
	global status EMsgText

	if { [regexp "^ *\{.*\} *$" $record] !=1 } then {
		return [procERROR "Invalid format. Message Text must be blank\"text\"."]
	} 
	set MsgText [lindex $record 0]

 	if { [string range $MsgText 0 21] != "\[%%hostx:%AGENT_ADDR%\]" } then {
		return [procERROR "Invalid message text. Message Text must be started with \"\[%%hostx:%AGENT_ADDR%\]\" ."]
	}
	if { [string is ascii $record ] == 0 } then {
		return [procERROR "Japanese character in English message text. "]
	}

	set status "WEVENTTYPE"
	regsub -all {\\\{} $MsgText \{ MsgText
	regsub -all {\\\}} $MsgText \} MsgText
	regsub -all {\\"}  $MsgText {""} MsgText
	set EMsgText $MsgText
	return 0


}


# 
# procWJMSGDATAL
# 
proc procWJMSGDATAL { record } {
	global status JMsgText

	if { [regexp "^ *\{.*\} *$" $record] !=1 } then {
		return [procERROR "Invalid format. Message Text must be blank\"text\"."]
	} 
	set MsgText [lindex $record 0]

 	if { [string range $MsgText 0 21] != "\[%%hostx:%AGENT_ADDR%\]" } then {
		return [procERROR "Invalid message text. Message Text must be started with \"\[%%hostx:%AGENT_ADDR%\]\" ."]
	}

	set status "WEVENTTYPE"
	regsub -all {\\\{} $MsgText \{ MsgText
	regsub -all {\\\}} $MsgText \} MsgText
	regsub -all {\\"}  $MsgText {""} MsgText
 	set JMsgText \"$MsgText\"
	return 0

}


# 
# procWEVENTTYPE
# 
proc procWEVENTTYPE { record } {
	global status

	if { [regexp "^ *EVENTTYPE:" $record ] != 1 && [regexp "^ *EventType:" $record ] != 1 } then {
		return [procERROR "Invalid keyword. \"EventType:\" required ."]
	}
 	if { [llength $record] > 1 } {
		return [procERROR "Invalid parameter specified. Keyword's parameter must be in the following record." ]
	}

	set status "WEVENTTYPEDATA"
	return 0
}


# 
# procWEVENTTYPEDATA
# 
proc procWEVENTTYPEDATA { record } {
	global status

	if { [llength $record] > 1 } then {
		return [procERROR "Invalid parameter number. Only 1 parameter must be in this record."]
	}
	if { [lindex $record 0]!="ERROR" && [lindex $record 0]!="WARNING" && [lindex $record 0]!="INFORMATION" && [lindex $record 0]!="AUDIT_SUCCESS" && [lindex $record 0]!="AUDIT_FAIL" } then {
		return [procERROR "Parameter invalid. It must be ERROR/WARNING/INFORMATION/AUDIT_SUCCESS/AUDIT_FAIL."]
	}

	set status "WTHRESHOLD"
	return 0
}
 

# 
# procWSOURCE
# 
proc procWSOURCE { record } {
	global status

	if { [regexp "^Source:" $record ] != 1 } then {
		return [procERROR "Invalid keyword. \"Source:\" required."]
	}
 	if { [llength $record 1] > 1 } {
		return [procERROR "Invalid parameter specified. Keyword's parameter must be in the following record." ]
	}

	set status "WSOURCEDATA"
	return 0
}


# 
# procWSOURCEDATA
# 
proc procWSOURCEDATA { record } {
	global status

	if { [llength $record] > 1 } then {
		return [procERROR "Invalid parameter number. Only 1 parameter must be in this record."]
	}
#	if { [lindex $record 0]!="MpCNappl" } then {
#		return [procERROR "Parameter invalid. It must be \"MpCNappl\"."]
#	}

	set status "WTHRESHOLD"
	return 0
}
 


# 
# procWTHRESHOLD
# 
proc procWTHRESHOLD { record } {
	global status

	if { [regexp -nocase "^ *SOURCE:" $record ] == 1 } then {
		set status "WSOURCEDATA"
		return 0;
	}
	if { [regexp -nocase "^ *Threshold:" $record ] != 1 } then {
		return [procERROR "Invalid keyword. \"Threshold:\" required."]
	}
 	if { [llength $record] > 1 } {
		return [procERROR "Invalid parameter specified. Keyword's parameter must be in the following record." ]
	}

	set status "WTHRESHOLDDATA"
	return 0
}


# 
# procWTHRESHOLDDATA
# 
proc procWTHRESHOLDDATA { record } {
	global status

	if { [llength $record] > 1 } then {
		return [procERROR "Invalid parameter number. Only 1 parameter must be in this record."]
	}
	if { [lindex $record 0]!="0" && [lindex $record 0]!="1" } then {
		return [procERROR "Parameter invalid. It must be 1 or 0."]
	}

	set status "WCMNT"
	return 0
}
 

# 
# procPUTOUT
# 
proc procPUTOUT { } {
	global JMsgText EMsgText VBINDEX Sourceline SWDir Generic Community Oid Agent
	global Specific	Mib Mtype Mvalue	

	# Count all Varbind data number
	# delete by S.Nakagoshi 2004/02/18 - start -
	#set  Jnext [regsub -all "%NEXT_V%" $JMsgText " " JMsgText]
	#incr Jnext
	#set  Jback [regsub -all "%BACK_V%" $JMsgText " " JMsgText]
	#
	#set  Enext [regsub -all "%NEXT_V%" $EMsgText " " EMsgText]
	#incr Enext
	#set  Eback [regsub -all "%BACK_V%" $EMsgText " " EMsgText]
	#
	#set vbindex [Max $VBINDEX [expr $Jnext - $Jback + 1] ]
	#set vbindex [Max $vbindex [expr $Enext - $Eback + 1] ]
	# delete by S.Nakagoshi 2004/02/18 - end -
	# add by S.Nakagoshi 2004/02/18 - start -
	set JVarbindCount [CountVarbind $JMsgText]
	set EVarbindCount [CountVarbind $EMsgText]
	set vbindex [Max $VBINDEX $JVarbindCount ]
	set vbindex [Max $vbindex $EVarbindCount ]
	# add by S.Nakagoshi 2004/02/18 - end -
		
	puts stdout [format {rem * Test Data for Source %s.} $Sourceline]
	puts -nonewline stdout $SWDir
	puts -nonewline stdout "\\mpwalker.dm\\mpnetmgr\\bin\\snmptrap"
	puts -nonewline stdout [format { -a %%DEST%% -g %d -c %s -E %s -A %s } $Generic $Community $Oid $Agent]
	if { $Generic == "6" && $Specific == "-" } then {
		puts -nonewline stdout [format {-s 12345 -h 1234566}]
	} elseif { $Specific != "-" } then {
		puts -nonewline stdout [format {-s %d -h 1234566} $Specific]
	}
#	if { $Generic  == "6" } then {
#		puts -nonewline stdout [format {-s %d -h 1234566} $Specific]
#	}
 	set index 0
 	while { $index < $vbindex } {
		if { $index > $VBINDEX } then {
			puts -nonewline stdout " -m sysDescr -o VarBind#[expr $index + 1]"
		} elseif { [array size Mib] <= $index } then {
			puts -nonewline stdout " -m sysDescr -o VarBind#[expr $index + 1]"
 		} elseif {$Mib($index) == "-" } then {
			puts -nonewline stdout " -m sysDescr -o VarBind#[expr $index + 1]"
 		} else {
			puts -nonewline stdout [format { -m %s %s %s} $Mib($index) [ConvTypes $Mtype($index)] $Mvalue($index) ]
 		}
 		incr index
	}
 	puts stdout ""
	puts stdout ""

	return
}

# add by S.Nakagoshi 2004/02/18
# 
# Count Varbind
# 
proc CountVarbind { string } {
	set cnt 1
	set max 1
	while { [regexp "(%NEXT_V%|%BACK_V%)" ${string} WORD] != 0 } {
		switch -- "${WORD}" {
			{%NEXT_V%} {
				incr cnt
			}
			{%BACK_V%} {
				incr cnt -1
				if { $cnt < 0 } {
					set cnt 0
				}
			}
			default {
			}
		}
		if { $cnt > $max } {
			set max $cnt
		}
		regsub ${WORD} ${string} "" string
	}
	return $max
}


# 
# Operands check
# 
proc OperandsCheck { c v } {
	global procSW inFile SWDir

	if { $c == 1 } then {
		set inputfile [lindex $v 0]
		set procSW "C"
	} elseif { $c == 2 } then {
		set inputfile [lindex $v 0]
		set procSW [lindex $v 1]
	} elseif { $c == 3 } then {
		set inputfile [lindex $v 0]
		set procSW [lindex $v 1]
		set SWDir  [lindex $v 2]
	} else {
		puts stderr "ERROR : Invalid parameter"
		exit 12
	}
		
	if { [ file readable $inputfile ] == 0 } then {
		puts stderr "ERROR : $inputfile is not readable"
		exit 12
	} else {
		set inFile [open $inputfile r]
		fconfigure $inFile -encoding shiftjis
 	}

#	if { $SWDir != "" } then {
#		regsub "\\$" $SWDir "" SWDir
#		if { [file exist [set SWDir]\\mpwalker.dm\\mpcnappl\\mpcnmgr\\bin\\cnsetcnfmg.exe] == 0 } then {
#			puts stderr "ERROR : Systemwalker Install Directory is invalid"
#			exit 12
#		}
#	}

	if { $procSW != "C" && $procSW != "DC" && $procSW != "DCT" } then { 
		puts stderr "ERROR : 2nd parameter must be C/DC/DCT"
		exit 12
	} elseif { $procSW != "C" && $SWDir == "" } then {
		puts stderr "ERROR : Systemwalker Install Directory not specified"
		exit 12
	}
	return 0

}


#
# ConvTypes
#
proc ConvTypes { in } {

	if { $in == "INT" } {return "-i"}
		if { $in == "OCT" } {return "-D"}
		if { $in == "OID" } {return "-d"}
		if { $in == "IPA" } {return "-I"}
		if { $in == "CNT" } {return "-C"}
		if { $in == "GAG" } {return "-G"}
		if { $in == "TIM" } {return "-T"}
		if { $in == "OPA" } {return "-O"}
}



# 
# InitProc
# 
proc InitProc { } {
	global procSW SWDir objname objdot objtype 

	puts stderr "******************************************\n"
	puts stderr " Check processing Start.\n"
	puts stderr "******************************************\n"

	if { $procSW == "C" } then { return 0 }

	if { $procSW == "DCT" } then {
		puts stdout "@echo off "
		puts stdout "setlocal"
		puts stdout "rem ##################################################### " 
		puts stdout "rem # " 
		puts stdout "rem # This is output of t2e.cnf syntax check tool. " 
		puts stdout "rem # This tool genarates test data " 
		puts stdout "rem # " 
		puts stdout "rem ##################################################### "
		puts stdout "" 
		puts stdout "if not \"%1\" == \"\" goto LVL1       "
		puts stdout "echo %0 DestAddr \[AgentAddr\]"
		puts stdout "goto END                      "
		puts stdout ":LVL1                         "
		puts stdout "set DEST=%1                   "
		puts stdout "if not \"%2\" == \"\" goto LVL2   "
		puts stdout "set SOURCE=%1                 "
		puts stdout "goto GO                       "
		puts stdout ":LVL2                         "
		puts stdout "set SOURCE=%2                 "
		puts stdout ":GO                           "
		puts stdout ""
	}
 
	set objname ""
	set objdot  ""
	set objtype ""
	
	set mibDir $SWDir
	append mibDir "\\mpwalker.dm\\mib" 
	# Read xxx.MIB file, and initialise obj name list, obj dot-format-id list, and obj type list.
	puts -nonewline stderr "Mib file reading."
	puts stderr ""
	# path separators are converted platform separator to tcl separator
	set mibDir [file join ${mibDir}]

	# getting mib file list
	set mibfiles [glob -path ${mibDir}/ {*.[mM][iI][bB]}]

	foreach file ${mibfiles} {
		# Read xxx.MIB file,
		# and initialise obj name list and obj dot-format-id list.
		set mfileid [open "${file}" r]
		while { [eof ${mfileid}] == 0 } {
			gets ${mfileid} record
			regsub -all "," ${record} " " record
			if { [lsearch -exact $objname [lindex ${record} 0] ] } {
				lappend objname [lindex $record 0]
				lappend objdot  [lindex $record 1]
				lappend objtype [lindex $record 2]
			}
		}
		close ${mfileid}
	}
	return 0
}





##################################################################
proc d2n {input} {

	global objname
	global objdot

	regsub -all {\.} $input " " idlist
	set numofids [llength $idlist]
	set index -1
	for { set i $numofids } { $i > 0 } { incr i -1} {
		set temptarget ""
		for { set j 0 } { $j < $i } { incr j } {
			if {$j!=0} { append temptarget "." }
			append temptarget [lindex $idlist $j]
		}
		set index [lsearch -exact $objdot $temptarget]
		if { $index > 0 } break 
	}
	if { $index > 0 } {
		set result [lindex $objname $index]
		for { set j $i } { $j < $numofids } { incr j } {
			append result "."
			append result [lindex $idlist $j]
		}
		return $result 
	} else {
		return ".not-match." 
	}
}


##################################################################
proc n2d {input} {

	global objname
	global objdot

	if { [isAllDot $input] == "DOT" } then {
		return $input
	}
	set index [ string first "." $input ]
	if { $index < 0 } {
		set tindex [lsearch $objname $input]
		if { $tindex > 0 } {
			set result [lindex $objdot $tindex]
			return $result

		} else {
			return ".not-match." 
		}
	} else {
		set namepart [string range $input 0 [expr $index-1 ] ]
		set dotpart  [string range $input $index end]
		if { [regexp {^[0-9\.]*$} $dotpart ] == 0 } {
			return "bad-format"
		} else {
			set tindex [lsearch $objname $namepart]
			if { $tindex > 0 } {
				set result [lindex $objdot $tindex]
				set alldot $result
				append alldot $dotpart				
				return $alldot
			} else {
				return ".not-match." 
			}
		}
	}
}


# 
# TermProc
# 
proc TermProc { } {
	global rCount status defCount

	set rcode 0
	if { $status != "WCMNT" && $status != "WFIRST" } then {
		set rcode [ Max $rcode [procERROR "Unexpected end of conversion definition caused by EOF or ERROR. " ]]
	}
	if { $rCount == 0 } then {
		set rcode [ Max $rcode [procWARN "There is no input record." ]]
	} elseif { $defCount == 0 } {
		set rcode [ Max $rcode [procWARN "There is no definition." ]]
	}

	puts stderr ""
	puts stderr "******************************************\n"
	puts stderr " Check processing has ended.\n"
	puts stderr "       Processed records     : $rCount\n"
	puts stderr "       Processed definitions : $defCount\n"
	puts stderr "******************************************\n"

	return $rcode
}

# 
# nextRecord
# 
proc nextRecord { } {
	global inFile rCount

	gets $inFile record
	if { [eof $inFile]==1 } {
		return ""
	} else {
		incr rCount
		return $record
	}
}

# 
# Rn
# 
proc Rn { } {
	global rCount

	return [format {%04d} $rCount]
}


# 
# Max
# 
proc Max { a b } {
	if { $a >= $b } then {
		return $a
	} else {
		return $b
	}
}


# 
# isExistMib
# 
proc isExistMib { input } {
	global objname objdot objtype

	# 2003/02/27 change logic isExistMib by S.Nakagoshi

	# ifTable.1.1.1 -> 1.3.6.1.2.1.2.2.1.1.1 
	set mibname [n2d $input]
	if { $mibname != ".not-match." } {
		# 1.3.6.1.2.1.2.2.1.1.1 -> ifIndex.1
		set mibname [d2n $mibname]
		if { $mibname != ".not-match." } {
			set input $mibname
		}
	}
	# specified all dot and numeric ?
	if { [isAllDot $input] == "DOT" } then {
		# input is make of dot and numeric
		set tindex [lsearch -exact $objdot $input]
		if { $tindex > 0 } {
			set result [lindex $objtype $tindex]
		} else {
			return "UKN" 
		}
	} else {
		set index [ string first "." $input ]
		if { $index < 0 } {
			# input has not "dot". ex) ifIndex
			set tindex [lsearch $objname $input]
			if { $tindex < 1 } {
				return "UKN" 
			}
			set result [lindex $objtype $tindex]
			if { $result == "n" } {
				# no "dot" and table object
				return "UKNT"
			}
		} else {
			# input has "dot". ex) ifIndex.1
			set namepart [string range $input 0 [expr $index-1 ] ]
			set dotpart  [string range $input [expr $index+1 ] end]
			if { [isAllDot $dotpart ] != "DOT" } {
				return "INV"
			}
			if { $namepart == "" } then {
				return "INV"
			}
			# search index of objname
			set tindex [lsearch $objname $namepart]
			if { $tindex > 0 } {
				set result [lindex $objtype $tindex]
			} else {
				return "UKN" 
			}
		}
	}

	if       { $result == "i" } then {
		return "INT"
	} elseif { $result == "d" } then { 
		return "OID" 
	} elseif { $result == "o" } then {
		return "OCT" 
	} elseif { $result == "a" } then { 
		return "IPA" 
 	} elseif { $result == "c" } then { 
		return "CNT" 
	} elseif { $result == "g" } then { 
		return "GAG" 
	} elseif { $result == "t" } then { 
		return "TIM" 
	} else {
		#return "UKNT"
		return "UKN"
	} 

}

 
# 
# isAllDot
# 
proc isAllDot { a } {

	if { [regexp {^[0-9][0-9\.]*$} $a] ==1 && [regexp {\.$} $a] !=1 && [regexp {\.\.} $a] !=1 } then {
		return "DOT"
		# "DOT"   ... Dot format 
	} else {
		return ""
		# ""      ... No 
	}
}
 


 
# 
# checkRecord
# 
proc checkRecord { } {

	global RawRecord

#	regsub -all \{ $RawRecord \\\{ RawRecord
#	regsub -all \} $RawRecord \\\} RawRecord
#	regsub -all {\[} $RawRecord {\\[} RawRecord
#	regsub -all {\]} $RawRecord {\\]} RawRecord

	# 2003/02/27 change logic checkRecord by S.Nakagoshi
	#puts stderr "before=$RawRecord"
	#regsub -all -expanded {\\\\([^\\])} $RawRecord {\\\1} RawRecord
	#puts stderr "after =$RawRecord"
	regsub -all {\\\\} $RawRecord {\\} RawRecord
	set stat "out"
	for { set i 0 } { $i<[string length $RawRecord]} {incr i} {
		if {$stat=="out"} then {
			if {[string index $RawRecord $i] == "\"" } then {
				set RawRecord [string replace $RawRecord $i $i \{ ]
				set stat "in"
			}
		} elseif {[string index $RawRecord $i] == "\"" } then {
			set RawRecord [string replace $RawRecord $i $i \} ]
			set stat "out"
		} elseif {[string index $RawRecord $i] == "\\" } then {
			if {[string index $RawRecord [expr $i +1] ] == "\"" } then {
				incr i
			}
		}
	}
	if { [catch {llength $RawRecord} len] } {
		return  [procERROR "Record format is invalid."]
	} else {
		return 0
	}
}
 


# 
# isIPA
# 
proc isIPA { a } {

	if { [isAllDot $a] == "" } then {
		return ""
		# ""   ... No 
	}
	regsub -all {\.} $a " " adlist
	if { [llength $adlist] != 4} then {
		return ""
		# ""   ... No 
	}
	foreach { ad }  $adlist  {
		if {  $ad >= 256 } then {
			return ""
			# ""   ... No 
		}
		if { [string length $ad] >= 4 } then {
			return ""
			# ""   ... No 
		}
	} 
	return "IPA"
	# "IPA"      ... IP address 
	
}
 


# 
# Main proc
# 

	# Gloval Variables
	variable status "WFIRST"
	variable inFile 0
	variable procSW "C"
	variable rCount 0
	variable defCount 0
	variable SWDir  ""

	variable Enterprise ""
	variable Sourceline 0
	variable Community ""
	variable Oid ""
	variable Agent""
	variable Generic ""
	variable Specific ""
	variable VBINDEX 0
	variable Mib 
	variable Mtype 
	variable Maorb 
	variable Mvalue 
	variable EMsgText ""
	variable JMsgText ""

	variable RawRecord

	variable topOID ""
	variable topOIDdot ""

	variable objtype ""
	variable objname ""
	variable objdot ""

	set lastECode 0
	set procErrorCode 0
	set tempcode 0

	# Operands check
	set procErrorCode [OperandsCheck $argc $argv]
	if { $procErrorCode > 4 } then { exit $procErrorCode }

	# Initialization
	InitProc

	set RawRecord [ nextRecord ]
	if { [eof $inFile]==1} then {
		# Termination
		set lastECode [Max $lastECode [TermProc] ]
		exit $lastECode
	} else	{
		set procErrorCode [ Max $procErrorCode [checkRecord] ]
		if { $procErrorCode == 0 } then {
			set procErrorCode [ Max $procErrorCode [procWFIRST $RawRecord] ]
		} else {
			set procErrorCode [procSERROR "First Record must be \"# enterprises=<enterprise OID>\" "]
		}
		
		set lastECode [Max $lastECode $procErrorCode ]
		set RawRecord [ nextRecord ]
	}
	while { [eof $inFile]==0 && $lastECode <= 8 } {
 		if { [regexp {^# trap -> event conversion setup file} $RawRecord] == 1} then {
			if { $status != "WCMNT" && $status != "ERRSTAT" } then {
				set lastECode [Max $lastECode [procERROR "Unexpected end of conversion definition."] ]
				set status "WCMNT"
				set procErrorCode 0
			}
			set procErrorCode [procWCMNT $RawRecord]
			set lastECode [Max $lastECode $procErrorCode ]
			incr defCount
		} elseif {[regexp {^[ @]*$} $RawRecord ] == 1 || [regexp {^#.*$} $RawRecord ] == 1} then {
			# skip this RawRecord

		} elseif { [set tempcode [checkRecord] ] >0 } then {
			set procErrorCode [Max $procErrorCode $tempcode]
			set lastECode [Max $lastECode $procErrorCode ]

		} elseif { $status == "WCMNT" } then {
			set procErrorCode [ Max $procErrorCode [procERROR "Unexpected record data."] ]
			set lastECode [Max $lastECode $procErrorCode ]

		} elseif { $status == "WTRAP" } then {
			set procErrorCode [ Max $procErrorCode [procWTRAP $RawRecord] ]
			set lastECode [Max $lastECode $procErrorCode ]
		} elseif { $status == "WTRAPDATA" } then {
			set procErrorCode [ Max $procErrorCode [procWTRAPDATA $RawRecord] ]
			set lastECode [Max $lastECode $procErrorCode ]

		} elseif { $status == "WVARBIND" } then {
			set procErrorCode [ Max $procErrorCode [procWVARBIND $RawRecord] ]
			set lastECode [Max $lastECode $procErrorCode ]
		} elseif { $status == "WVARBINDDATA" || $status == "WVARBINDDATAORMESSAGE" } then {
			set procErrorCode [ Max $procErrorCode [procWVARBINDDATA $RawRecord] ]
			set lastECode [Max $lastECode $procErrorCode ]

		} elseif { $status == "WEJMSG" } then {
			set procErrorCode [ Max $procErrorCode [procWEJMSG $RawRecord] ]
			set lastECode [Max $lastECode $procErrorCode ]
		} elseif { $status == "WEMSGDATAF" } then {
			set procErrorCode [ Max $procErrorCode [procWEMSGDATAF $RawRecord] ]
			set lastECode [Max $lastECode $procErrorCode ]
		} elseif { $status == "WJMSGDATAF" } then {
			set procErrorCode [ Max $procErrorCode [procWJMSGDATAF $RawRecord] ]
			set lastECode [Max $lastECode $procErrorCode ]
		} elseif { $status == "WEMSGL" } then {
			set procErrorCode [ Max $procErrorCode [procWEMSGL $RawRecord] ]
			set lastECode [Max $lastECode $procErrorCode ]
		} elseif { $status == "WJMSGL" } then {
			set procErrorCode [ Max $procErrorCode [procWJMSGL $RawRecord] ]
			set lastECode [Max $lastECode $procErrorCode ]
		} elseif { $status == "WEMSGDATAL" } then {
			set procErrorCode [ Max $procErrorCode [procWEMSGDATAL $RawRecord] ]
			set lastECode [Max $lastECode $procErrorCode ]
		} elseif { $status == "WJMSGDATAL" } then {
			set procErrorCode [ Max $procErrorCode [procWJMSGDATAL $RawRecord] ]
			set lastECode [Max $lastECode $procErrorCode ]

		} elseif { $status == "WEVENTTYPE" } then {
			set procErrorCode [ Max $procErrorCode [procWEVENTTYPE $RawRecord] ]
			set lastECode [Max $lastECode $procErrorCode ]
		} elseif { $status == "WEVENTTYPEDATA" } then {
			set procErrorCode [ Max $procErrorCode [procWEVENTTYPEDATA $RawRecord] ]
			set lastECode [Max $lastECode $procErrorCode ]

		} elseif { $status == "WSOURCE" } then {
			if {[regexp {^ *Threshold:} $RawRecord ] == 1} then {
				set status "WTHRESHOLD"
				continue
			}
			set procErrorCode [ Max $procErrorCode [procWSOURCE $RawRecord] ]
			set lastECode [Max $lastECode $procErrorCode ]
		} elseif { $status == "WSOURCEDATA" } then {
			set procErrorCode [ Max $procErrorCode [procWSOURCEDATA $RawRecord] ]
			set lastECode [Max $lastECode $procErrorCode ]

		} elseif { $status == "WTHRESHOLD" } then {
			set procErrorCode [ Max $procErrorCode [procWTHRESHOLD $RawRecord] ]
			set lastECode [Max $lastECode $procErrorCode ]
		} elseif { $status == "WTHRESHOLDDATA" } then {
			set procErrorCode [ Max $procErrorCode [procWTHRESHOLDDATA $RawRecord] ]
			if { $procSW == "DCT" && $procErrorCode <= 4 } {
		 		# Syntacs check ended.
 				# And there is no Error.
				procPUTOUT
			}
			set status "WCMNT"
			set lastECode [ Max $lastECode $procErrorCode ]
			set procErrorCode 0
		} elseif { $status == "ERRSTAT" } then {
			if {[regexp "^TRAP:" $RawRecord ] == 1} then {
				set status "WTRAP"
				continue
			} elseif {[regexp "^VARBIND:" $RawRecord ] == 1} then {
				set status "WVARBIND"
				continue
			} elseif {[regexp "^MESSAGE:" $RawRecord ] == 1} then {
				set status "WVARBINDDATAORMESSAGE"
				continue
			} elseif {[regexp "^Threshold:" $RawRecord ] == 1} then {
				set status "WTHRESHOLD"
				continue
			} elseif {[regexp "^EventType:" $RawRecord ] == 1} then {
				set status "WEVENTTYPE"
				continue
			} elseif {[regexp "^EVENTTYPE:" $RawRecord ] == 1} then {
				set status "WEVENTTYPE"
				continue
			} else {
				procWARN "Skipped by error detection."
			}
		}
		if { $lastECode <= 8 } { set RawRecord [ nextRecord ] }
	}	

	# Termination
	set lastECode [Max $lastECode [TermProc] ]

	exit $lastECode

